# This file was auto-generated by Fern from our API Definition.

import contextlib
import typing
from json.decoder import JSONDecodeError
from logging import error, warning

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.http_sse._api import EventSource
from ...core.pydantic_utilities import parse_sse_obj
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.lightningv2request_language import Lightningv2RequestLanguage
from ..types.lightningv2request_output_format import Lightningv2RequestOutputFormat

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLightningV2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.contextmanager
    def synthesize_lightningv2speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[int] = OMIT,
        speed: typing.Optional[float] = OMIT,
        consistency: typing.Optional[float] = OMIT,
        similarity: typing.Optional[float] = OMIT,
        enhancement: typing.Optional[float] = OMIT,
        language: typing.Optional[Lightningv2RequestLanguage] = OMIT,
        output_format: typing.Optional[Lightningv2RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Get speech for given text using the Waves API

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[int]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        consistency : typing.Optional[float]
            This parameter controls word repetition and skipping. Decrease it to prevent skipped words, and increase it to prevent repetition.

        similarity : typing.Optional[float]
            This parameter controls the similarity between the generated speech and the reference audio. Increase it to make the speech more similar to the reference audio.

        enhancement : typing.Optional[float]
            Enhances speech quality at the cost of increased latency.

        language : typing.Optional[Lightningv2RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read as individual digits in English. If set to 'hi', numbers will be read as individual digits in Hindi.

        output_format : typing.Optional[Lightningv2RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            Synthesized speech retrieved successfully.
        """
        with self._client_wrapper.httpx_client.stream(
            "api/v1/lightning-v2/get_speech",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            json={
                "text": text,
                "voice_id": voice_id,
                "sample_rate": sample_rate,
                "speed": speed,
                "consistency": consistency,
                "similarity": similarity,
                "enhancement": enhancement,
                "language": language,
                "output_format": output_format,
                "pronunciation_dicts": pronunciation_dicts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", None) if request_options is not None else None
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise InternalServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()

    @contextlib.contextmanager
    def stream_lightningv2speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[int] = OMIT,
        speed: typing.Optional[float] = OMIT,
        consistency: typing.Optional[float] = OMIT,
        similarity: typing.Optional[float] = OMIT,
        enhancement: typing.Optional[float] = OMIT,
        language: typing.Optional[Lightningv2RequestLanguage] = OMIT,
        output_format: typing.Optional[Lightningv2RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[typing.Any]]]:
        """
        The Lightning v2 SSE API provides real-time text-to-speech streaming capabilities with high-quality voice synthesis. This API uses Server-Sent Events (SSE) to deliver audio chunks as they're generated, enabling low-latency audio playback without waiting for the entire audio file to process.
        For an end-to-end example of how to use the Lightning v2 SSE API, check out [Text to Speech (SSE) Example](https://github.com/smallest-inc/waves-examples/blob/main/lightning_v2/http_streaming/http_streaming_api.py)

        ## When to Use

        - **Interactive Applications**: Perfect for chatbots, virtual assistants, and other applications requiring immediate voice responses
        - **Long-Form Content**: Efficiently stream audio for articles, stories, or other long-form content without buffering delays
        - **Voice User Interfaces**: Create natural-sounding voice interfaces with minimal perceived latency
        - **Accessibility Solutions**: Provide real-time audio versions of written content for users with visual impairments

        ## How It Works

        1. **Make a POST Request**: Send your text and voice settings to the API endpoint
        2. **Receive Audio Chunks**: The API processes your text and streams audio back as base64-encoded chunks with 1024 byte size
        3. **Process the Stream**: Handle the SSE events to decode and play audio chunks sequentially
        4. **End of Stream**: The API sends a completion event when all audio has been delivered

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[int]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        consistency : typing.Optional[float]
            This parameter controls word repetition and skipping. Decrease it to prevent skipped words, and increase it to prevent repetition.

        similarity : typing.Optional[float]
            This parameter controls the similarity between the generated speech and the reference audio. Increase it to make the speech more similar to the reference audio.

        enhancement : typing.Optional[float]
            Enhances speech quality at the cost of increased latency.

        language : typing.Optional[Lightningv2RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read as individual digits in English. If set to 'hi', numbers will be read as individual digits in Hindi.

        output_format : typing.Optional[Lightningv2RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[HttpResponse[typing.Iterator[typing.Any]]]
            Synthesized speech retrieved successfully.
        """
        with self._client_wrapper.httpx_client.stream(
            "api/v1/lightning-v2/stream",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            json={
                "text": text,
                "voice_id": voice_id,
                "sample_rate": sample_rate,
                "speed": speed,
                "consistency": consistency,
                "similarity": similarity,
                "enhancement": enhancement,
                "language": language,
                "output_format": output_format,
                "pronunciation_dicts": pronunciation_dicts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[typing.Any]]:
                try:
                    if 200 <= _response.status_code < 300:

                        def _iter():
                            _event_source = EventSource(_response)
                            for _sse in _event_source.iter_sse():
                                if _sse.data == None:
                                    return
                                try:
                                    yield typing.cast(
                                        typing.Any,
                                        parse_sse_obj(
                                            sse=_sse,
                                            type_=typing.Any,  # type: ignore
                                        ),
                                    )
                                except JSONDecodeError as e:
                                    warning(f"Skipping SSE event with invalid JSON: {e}, sse: {_sse!r}")
                                except (TypeError, ValueError, KeyError, AttributeError) as e:
                                    warning(
                                        f"Skipping SSE event due to model construction error: {type(e).__name__}: {e}, sse: {_sse!r}"
                                    )
                                except Exception as e:
                                    error(
                                        f"Unexpected error processing SSE event: {type(e).__name__}: {e}, sse: {_sse!r}"
                                    )
                            return

                        return HttpResponse(response=_response, data=_iter())
                    _response.read()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise InternalServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()


class AsyncRawLightningV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.asynccontextmanager
    async def synthesize_lightningv2speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[int] = OMIT,
        speed: typing.Optional[float] = OMIT,
        consistency: typing.Optional[float] = OMIT,
        similarity: typing.Optional[float] = OMIT,
        enhancement: typing.Optional[float] = OMIT,
        language: typing.Optional[Lightningv2RequestLanguage] = OMIT,
        output_format: typing.Optional[Lightningv2RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Get speech for given text using the Waves API

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[int]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        consistency : typing.Optional[float]
            This parameter controls word repetition and skipping. Decrease it to prevent skipped words, and increase it to prevent repetition.

        similarity : typing.Optional[float]
            This parameter controls the similarity between the generated speech and the reference audio. Increase it to make the speech more similar to the reference audio.

        enhancement : typing.Optional[float]
            Enhances speech quality at the cost of increased latency.

        language : typing.Optional[Lightningv2RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read as individual digits in English. If set to 'hi', numbers will be read as individual digits in Hindi.

        output_format : typing.Optional[Lightningv2RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            Synthesized speech retrieved successfully.
        """
        async with self._client_wrapper.httpx_client.stream(
            "api/v1/lightning-v2/get_speech",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            json={
                "text": text,
                "voice_id": voice_id,
                "sample_rate": sample_rate,
                "speed": speed,
                "consistency": consistency,
                "similarity": similarity,
                "enhancement": enhancement,
                "language": language,
                "output_format": output_format,
                "pronunciation_dicts": pronunciation_dicts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", None) if request_options is not None else None
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise InternalServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()

    @contextlib.asynccontextmanager
    async def stream_lightningv2speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[int] = OMIT,
        speed: typing.Optional[float] = OMIT,
        consistency: typing.Optional[float] = OMIT,
        similarity: typing.Optional[float] = OMIT,
        enhancement: typing.Optional[float] = OMIT,
        language: typing.Optional[Lightningv2RequestLanguage] = OMIT,
        output_format: typing.Optional[Lightningv2RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[typing.Any]]]:
        """
        The Lightning v2 SSE API provides real-time text-to-speech streaming capabilities with high-quality voice synthesis. This API uses Server-Sent Events (SSE) to deliver audio chunks as they're generated, enabling low-latency audio playback without waiting for the entire audio file to process.
        For an end-to-end example of how to use the Lightning v2 SSE API, check out [Text to Speech (SSE) Example](https://github.com/smallest-inc/waves-examples/blob/main/lightning_v2/http_streaming/http_streaming_api.py)

        ## When to Use

        - **Interactive Applications**: Perfect for chatbots, virtual assistants, and other applications requiring immediate voice responses
        - **Long-Form Content**: Efficiently stream audio for articles, stories, or other long-form content without buffering delays
        - **Voice User Interfaces**: Create natural-sounding voice interfaces with minimal perceived latency
        - **Accessibility Solutions**: Provide real-time audio versions of written content for users with visual impairments

        ## How It Works

        1. **Make a POST Request**: Send your text and voice settings to the API endpoint
        2. **Receive Audio Chunks**: The API processes your text and streams audio back as base64-encoded chunks with 1024 byte size
        3. **Process the Stream**: Handle the SSE events to decode and play audio chunks sequentially
        4. **End of Stream**: The API sends a completion event when all audio has been delivered

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[int]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        consistency : typing.Optional[float]
            This parameter controls word repetition and skipping. Decrease it to prevent skipped words, and increase it to prevent repetition.

        similarity : typing.Optional[float]
            This parameter controls the similarity between the generated speech and the reference audio. Increase it to make the speech more similar to the reference audio.

        enhancement : typing.Optional[float]
            Enhances speech quality at the cost of increased latency.

        language : typing.Optional[Lightningv2RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read as individual digits in English. If set to 'hi', numbers will be read as individual digits in Hindi.

        output_format : typing.Optional[Lightningv2RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[typing.Any]]]
            Synthesized speech retrieved successfully.
        """
        async with self._client_wrapper.httpx_client.stream(
            "api/v1/lightning-v2/stream",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            json={
                "text": text,
                "voice_id": voice_id,
                "sample_rate": sample_rate,
                "speed": speed,
                "consistency": consistency,
                "similarity": similarity,
                "enhancement": enhancement,
                "language": language,
                "output_format": output_format,
                "pronunciation_dicts": pronunciation_dicts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[typing.Any]]:
                try:
                    if 200 <= _response.status_code < 300:

                        async def _iter():
                            _event_source = EventSource(_response)
                            async for _sse in _event_source.aiter_sse():
                                if _sse.data == None:
                                    return
                                try:
                                    yield typing.cast(
                                        typing.Any,
                                        parse_sse_obj(
                                            sse=_sse,
                                            type_=typing.Any,  # type: ignore
                                        ),
                                    )
                                except JSONDecodeError as e:
                                    warning(f"Skipping SSE event with invalid JSON: {e}, sse: {_sse!r}")
                                except (TypeError, ValueError, KeyError, AttributeError) as e:
                                    warning(
                                        f"Skipping SSE event due to model construction error: {type(e).__name__}: {e}, sse: {_sse!r}"
                                    )
                                except Exception as e:
                                    error(
                                        f"Unexpected error processing SSE event: {type(e).__name__}: {e}, sse: {_sse!r}"
                                    )
                            return

                        return AsyncHttpResponse(response=_response, data=_iter())
                    await _response.aread()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise InternalServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

# Backward-compat: v4.3.1 had atoms/audience.py with Audience class
from smallestai.atoms.helpers.audience import Audience  # noqa: F401




"""
Smallest AI Atoms Agent SDK.

A real-time WebSocket agent framework for building voice agents,
chatbots, and other interactive AI applications.

Usage:
    from smallestai.atoms.agent.server import AtomsApp
    from smallestai.atoms.agent.session import AgentSession
    from smallestai.atoms.agent.nodes import OutputAgentNode, BackgroundAgentNode
    from smallestai.atoms.agent.clients.openai import OpenAIClient
    from smallestai.atoms.agent.tools import function_tool, ToolRegistry
"""



# This file was auto-generated by Fern from our API Definition.

import typing

Lightningv2RequestLanguage = typing.Union[
    typing.Literal[
        "en", "hi", "ta", "kn", "mr", "bn", "gu", "ar", "he", "fr", "de", "pl", "ru", "it", "nl", "es", "sv", "ml", "te"
    ],
    typing.Any,
]



import asyncio
import uuid
from typing import Awaitable, Callable, Dict

import uvicorn
from fastapi import FastAPI, WebSocket
from loguru import logger

from smallestai.atoms.agent.session import AgentSession


class SessionHandler:
    """Manages all active WebSocket sessions"""

    def __init__(self):
        self._sessions: Dict[str, AgentSession] = {}

    async def create_session(
        self,
        websocket: WebSocket,
        setup_handler: Callable[[AgentSession], Awaitable[None]],
    ) -> str:
        """
        Create and start a new session.

        Args:
            websocket: WebSocket connection
            session_id: Unique session ID
            setup_handler: Async function to setup the session graph
        """
        # Create session
        session_id = f"session-{uuid.uuid4()}"
        session = AgentSession(
            websocket=websocket, session_id=session_id, setup_handler=setup_handler
        )
        await session.initialize()

        self._sessions[session_id] = session

        return session_id

    def disconnect(self, session_id: str) -> None:
        """Remove a session"""
        if session_id in self._sessions:
            del self._sessions[session_id]
            logger.debug(f"Removed session {session_id}")

    @property
    def active_sessions(self) -> int:
        """Get number of active sessions"""
        return len(self._sessions)

    async def shutdown(self) -> None:
        """Shutdown all sessions"""
        if self._sessions:
            logger.info(f"Closing {len(self._sessions)} active sessions")
            await asyncio.gather(
                *[session.cleanup() for session in self._sessions.values()],
                return_exceptions=True,
            )

            self._sessions.clear()


class AtomsApp:
    """
    FastAPI-based WebSocket server for Atoms SDK.

    Each incoming connection creates a new session with agent graph.

    Example:
        >>> from sdk.server import AtomsApp
        >>> from sdk.nodes.base_node import Node
        >>>
        >>> async def setup_session(session: AgentSession):
        ...     # Add nodes
        ...     node1 = MyNode()
        ...     session.add_node(node1)
        ...
        ...     # Add edges if multiple nodes
        ...     # session.add_edge(node1, node2)
        >>>
        >>> app = AtomsApp(setup_handler=setup_session)
        >>> app.run(port=8080)
    """

    def __init__(
        self,
        setup_handler: Callable[[AgentSession], Awaitable[None]],
    ):
        """
        Initialize Atoms WebSocket server.

        Args:
            setup_handler: Async function to setup session graph
            host: Host to bind to
            port: Port to listen on
        """
        self._host = "0.0.0.0"
        self._port = 8080
        self.setup_handler = setup_handler

        self.app = FastAPI(title="Atoms SDK Server", version="0.1.0")
        self.session_handler = SessionHandler()

        self._setup_routes()

    def _setup_routes(self) -> None:
        """Setup FastAPI routes"""

        @self.app.get("/")
        async def root():
            """Health check endpoint"""
            return {
                "status": "running",
                "active_sessions": self.session_handler.active_sessions,
            }

        @self.app.get("/health")
        async def health():
            """Health check endpoint"""
            return {
                "status": "ok",
                "active_sessions": self.session_handler.active_sessions,
            }

        @self.app.websocket("/ws")
        async def websocket_endpoint(websocket: WebSocket):
            """Main WebSocket endpoint for agent sessions"""

            session_id = None

            try:
                session_id = await self.session_handler.create_session(
                    websocket=websocket,
                    setup_handler=self.setup_handler,
                )
            finally:
                if session_id:
                    self.session_handler.disconnect(session_id=session_id)

        @self.app.on_event("shutdown")
        async def shutdown_event():
            """Cleanup on server shutdown"""
            logger.info("Server shutting down...")
            await self.session_handler.shutdown()

    def run(self, **kwargs) -> None:
        """
        Run the server (blocking).

        Args:
            **kwargs: Additional arguments to pass to uvicorn.run()
        """
        config = {
            "host": self._host,
            "port": self._port,
            "log_level": "info",
            **kwargs,
        }

        logger.info(f"Starting Atoms SDK server on ws://{self._host}:{self._port}/ws")
        uvicorn.run(self.app, **config)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from .types.post_conversation_outbound_response import PostConversationOutboundResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCallsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def start_an_outbound_call(
        self,
        *,
        agent_id: str,
        phone_number: str,
        variables: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        from_product_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PostConversationOutboundResponse]:
        """
        Initiates an outbound conversation with a specified agent and phone number.

        Parameters
        ----------
        agent_id : str
            The ID of the agent initiating the conversation

        phone_number : str
            The phone number to call

        variables : typing.Optional[typing.Dict[str, typing.Any]]
            The variables to pass to the conversation

        from_product_id : typing.Optional[str]
            The ID of the product to use for the call (get this from the /product/phone-numbers endpoint)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostConversationOutboundResponse]
            Successfully started the outbound conversation
        """
        _response = self._client_wrapper.httpx_client.request(
            "conversation/outbound",
            base_url=self._client_wrapper.get_environment().atoms,
            method="POST",
            json={
                "agentId": agent_id,
                "phoneNumber": phone_number,
                "variables": variables,
                "fromProductId": from_product_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostConversationOutboundResponse,
                    construct_type(
                        type_=PostConversationOutboundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCallsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def start_an_outbound_call(
        self,
        *,
        agent_id: str,
        phone_number: str,
        variables: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        from_product_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PostConversationOutboundResponse]:
        """
        Initiates an outbound conversation with a specified agent and phone number.

        Parameters
        ----------
        agent_id : str
            The ID of the agent initiating the conversation

        phone_number : str
            The phone number to call

        variables : typing.Optional[typing.Dict[str, typing.Any]]
            The variables to pass to the conversation

        from_product_id : typing.Optional[str]
            The ID of the product to use for the call (get this from the /product/phone-numbers endpoint)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostConversationOutboundResponse]
            Successfully started the outbound conversation
        """
        _response = await self._client_wrapper.httpx_client.request(
            "conversation/outbound",
            base_url=self._client_wrapper.get_environment().atoms,
            method="POST",
            json={
                "agentId": agent_id,
                "phoneNumber": phone_number,
                "variables": variables,
                "fromProductId": from_product_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostConversationOutboundResponse,
                    construct_type(
                        type_=PostConversationOutboundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



from pathlib import Path
from typing import Optional

import tomli
import tomli_w


class ProjectConfig:
    """Manages project-level configuration in .smallest/config.toml"""

    def __init__(self, project_dir: Optional[Path] = None):
        self.project_dir = project_dir or Path.cwd()
        self.config_dir = self.project_dir / ".smallestai"
        self.config_file = self.config_dir / "config.toml"

    def get_agent_id(self) -> Optional[str]:
        """Get agent_id from config, or None if not set."""
        config = self._load_config()
        return config.get("agent_id")

    def set_agent_id(self, agent_id: str) -> None:
        """Set agent_id in config."""
        config = self._load_config()
        config["agent_id"] = agent_id
        self._save_config(config)

    def _load_config(self) -> dict:
        if not self.config_file.exists():
            return {}
        try:
            with open(self.config_file, "rb") as f:
                return tomli.load(f)
        except Exception:
            return {}

    def _save_config(self, config: dict) -> None:
        self.config_dir.mkdir(parents=True, exist_ok=True)
        with open(self.config_file, "wb") as f:
            tomli_w.dump(config, f)



import asyncio
import json
from datetime import datetime
from enum import Enum
from typing import Any, ClassVar, Dict, List, Literal, Optional, Tuple

from pydantic import BaseModel, ConfigDict, Field
from rich.console import Console
from rich.live import Live
from rich.markdown import Markdown
from rich.panel import Panel
from rich.prompt import Prompt
from rich.text import Text
from websockets.asyncio.client import connect as websocket_connect

console = Console()


class EventDirection(str, Enum):
    """Direction of an event"""

    DOWNSTREAM = "downstream"
    UPSTREAM = "upstream"


class TypedModel(BaseModel):
    _subtypes_: ClassVar[List[Tuple[Any, Any]]] = []

    def __init_subclass__(cls, type=None):  # type: ignore
        cls._subtypes_.append((type, cls))

    @classmethod
    def get_cls(_cls, type):
        for t, cls in _cls._subtypes_:
            if t == type:
                return cls
        raise ValueError(f"Unknown type {type}")

    @classmethod
    def get_type(_cls, cls_name):
        for t, cls in _cls._subtypes_:
            if cls.__name__ == cls_name:
                return t
        raise ValueError(f"Unknown class {cls_name}")

    @classmethod
    def parse_obj(cls, obj):
        data_type = obj.get("type")
        if data_type is None:
            raise ValueError(f"type is required for {cls.__name__}")

        sub = cls.get_cls(data_type)
        if sub is None:
            raise ValueError(f"Unknown type {data_type}")
        return sub(**obj)

    @property
    def type(self):
        return self.get_type(self.__class__.__name__)


class ConversationType(str, Enum):
    TELEPHONY_INBOUND = "telephonyInbound"
    TELEPHONY_OUTBOUND = "telephonyOutbound"
    CHAT = "chat"
    WEBCALL = "webcall"
    SUPERVISOR_AGENT = "supervisor_agent"


class SessionContext(BaseModel):
    initial_variables: Dict[str, Any]
    conversation_type: ConversationType


class OutputAgentSettings(BaseModel):
    interruptible: bool = True

    model_config = ConfigDict(extra="allow")


class EventType(str, Enum):
    """SDKEvent type enumeration"""

    BASE = "base"

    # system events
    SYSTEM_BASE = "system.base"
    SYSTEM_INIT = "system.init"
    SYSTEM_LLM_REQUEST = "system.llm.request"
    SYSTEM_USER_STARTED_SPEAKING = "system.user.started_speaking"
    SYSTEM_USER_STOPPED_SPEAKING = "system.user.stopped_speaking"
    SYSTEM_CONTROL_INTERRUPT = "system.control.interrupt"
    SYSTEM_UPDATE_OUTPUT_AGENT_SETTINGS = "system.output_agent.update_settings"
    SYSTEM_USER_JOINED = "system.user.joined"

    # agent events
    AGENT_BASE = "agent.base"
    AGENT_READY = "agent.ready"
    AGENT_ERROR = "agent.error"
    AGENT_LLM_RESPONSE_START = "agent.llm.response.start"
    AGENT_LLM_RESPONSE_CHUNK = "agent.llm.response.chunk"
    AGENT_LLM_RESPONSE_END = "agent.llm.response.end"
    AGENT_CONTROL_INTERRUPT = "agent.control.interrupt"
    AGENT_CONTROL_MUTE_USER = "agent.control.mute_user"
    AGENT_CONTROL_UNMUTE_USER = "agent.control.unmute_user"
    AGENT_TRANSCRIPT_UPDATE = "agent.transcript.update"
    AGENT_SPEAK = "agent.speak"

    AGENT_END_CALL = "agent.end_call"
    AGENT_TRANSFER_CONVERSATION = "agent.transfer_call"


class SDKEvent(TypedModel, type=EventType.BASE.value):
    timestamp: datetime = Field(default_factory=datetime.now)


class SDKSystemEvent(SDKEvent, type=EventType.SYSTEM_BASE.value):
    pass


class SDKAgentEvent(SDKEvent, type=EventType.AGENT_BASE.value):
    pass


class SDKAgentTranscriptUpdateEvent(
    SDKAgentEvent, type=EventType.AGENT_TRANSCRIPT_UPDATE.value
):
    role: Literal["user", "assistant"]
    content: str


class SDKAgentEndCallEvent(SDKAgentEvent, type=EventType.AGENT_END_CALL.value):
    pass


class TransferOptionType(str, Enum):
    COLD_TRANSFER = "cold_transfer"
    WARM_TRANSFER = "warm_transfer"


class WarmTransferHandoffOptionType(str, Enum):
    PROMPT = "prompt"
    STATIC = "static"


class WarmTransferPrivateHandoffOption(BaseModel):
    """A model for the warm transfer private handoff option."""

    type: WarmTransferHandoffOptionType
    prompt: str


class WarmTransferPublicHandoffOption(BaseModel):
    """A model for the warm transfer public handoff option."""

    type: WarmTransferHandoffOptionType
    prompt: str


class TransferOption(BaseModel):
    """A model for the transfer option."""

    model_config = ConfigDict(populate_by_name=True)

    type: TransferOptionType = Field(default=TransferOptionType.COLD_TRANSFER)
    private_handoff_option: Optional[WarmTransferPrivateHandoffOption] = Field(
        default=None, alias="privateHandoffOption"
    )
    public_handoff_option: Optional[WarmTransferPublicHandoffOption] = Field(
        default=None, alias="publicHandoffOption"
    )


class SDKAgentTransferConversationEvent(
    SDKAgentEvent, type=EventType.AGENT_TRANSFER_CONVERSATION.value
):
    transfer_call_number: str
    transfer_options: TransferOption
    on_hold_music: Optional[
        Literal["ringtone", "relaxing_sound", "uplifting_beats", "none"]
    ]


class SDKSystemInitEvent(SDKSystemEvent, type=EventType.SYSTEM_INIT.value):
    version: str
    session_context: SessionContext
    output_agent_settings: Optional[OutputAgentSettings] = None


class SDKSystemUpdateOutputAgentSettingsEvent(
    SDKSystemEvent, type=EventType.SYSTEM_UPDATE_OUTPUT_AGENT_SETTINGS.value
):
    settings: Dict[str, Any]


class SDKSystemUserStartedSpeakingEvent(
    SDKSystemEvent, type=EventType.SYSTEM_USER_STARTED_SPEAKING.value
):
    pass


class SDKSystemUserStoppedSpeakingEvent(
    SDKSystemEvent, type=EventType.SYSTEM_USER_STOPPED_SPEAKING.value
):
    pass


class SDKSystemLLMRequestEvent(SDKSystemEvent, type=EventType.SYSTEM_LLM_REQUEST.value):
    """Request to LLM for completion."""

    extra_params: Dict[str, Any] = Field(default_factory=dict)


class SDKSystemControlInterruptEvent(
    SDKSystemEvent, type=EventType.SYSTEM_CONTROL_INTERRUPT.value
):
    pass


class SDKSystemUserJoinedEvent(SDKSystemEvent, type=EventType.SYSTEM_USER_JOINED.value):
    pass


class SDKAgentReadyEvent(SDKAgentEvent, type=EventType.AGENT_READY.value):
    pass


class SDKAgentErrorEvent(SDKAgentEvent, type=EventType.AGENT_ERROR.value):
    message: str


class SDKAgentLLMResponseStartEvent(
    SDKAgentEvent, type=EventType.AGENT_LLM_RESPONSE_START.value
):
    """Streaming response started."""

    request_id: Optional[str] = None


class SDKAgentLLMResponseChunkEvent(
    SDKAgentEvent, type=EventType.AGENT_LLM_RESPONSE_CHUNK.value
):
    text: str


class SDKAgentLLMResponseEndEvent(
    SDKAgentEvent, type=EventType.AGENT_LLM_RESPONSE_END.value
):
    pass


class SDKAgentControlInterruptEvent(
    SDKAgentEvent, type=EventType.AGENT_CONTROL_INTERRUPT.value
):
    pass


class SDKAgentControlMuteUserEvent(
    SDKAgentEvent, type=EventType.AGENT_CONTROL_MUTE_USER.value
):
    pass


class SDKAgentControlUnmuteUserEvent(
    SDKAgentEvent, type=EventType.AGENT_CONTROL_UNMUTE_USER.value
):
    pass


class SDKAgentSpeakEvent(SDKAgentEvent, type=EventType.AGENT_SPEAK.value):
    text: str


class SDKCodec:
    """
    Codec for encoding/decoding SDK Events.

    Handles bi-directional conversion:
    - Event → JSON bytes (for sending to SDK)
    - JSON bytes → Event (for receiving from SDK)
    """

    def encode(self, data: SDKEvent) -> bytes:
        """
        Encode SDK Event to JSON bytes for sending over WebSocket.

        Args:
            data: SDK Event to encode

        Returns:
            JSON bytes
        """
        return json.dumps(
            {
                "type": data.type,
                **data.model_dump(mode="json"),
            }
        ).encode()

    def decode(self, data: bytes) -> SDKEvent:
        """
        Decode JSON bytes to SDK Event.

        Args:
            data: JSON bytes from WebSocket

        Returns:
            SDK Event instance
        """
        if isinstance(data, str):
            data = data.encode()
        return SDKEvent.parse_obj(json.loads(data))


class ChatClient:
    """WebSocket client for chatting with Atoms SDK agents."""

    def __init__(self, ws_url: str):
        self.ws_url = ws_url
        self._websocket = None
        self._connected = False
        self.codec = SDKCodec()

    async def _connect(self) -> bool:
        """
        Connect to WebSocket server.

        Pattern: Modern websockets.asyncio.client.connect
        """
        try:
            self._websocket = await websocket_connect(
                self.ws_url,
            )

            await self._websocket.send(
                self.codec.encode(
                    SDKSystemInitEvent(
                        session_context=SessionContext(
                            initial_variables={},
                            conversation_type=ConversationType.CHAT,
                        ),
                        version="1.0.0",
                    )
                )
            )

            received_event = await asyncio.wait_for(
                self._websocket.recv(), timeout=10.0
            )
            if isinstance(received_event, str):
                received_event = received_event.encode()
            received_event = self.codec.decode(received_event)

            if not isinstance(received_event, SDKAgentReadyEvent):
                if isinstance(received_event, SDKAgentErrorEvent):
                    console.print(f"[red]Error: {received_event.message}[/red]")
                    return False
                else:
                    console.print(
                        f"[yellow]Unexpected event: {received_event.type}[/yellow]"
                    )
                    return False

            self._connected = True
            console.print("[green]✓ Connected and ready![/green]\n")
            return True

        except asyncio.TimeoutError:
            console.print("[red]Init timed out[/red]")
            return False

        except Exception as e:
            console.print(f"[red]Failed to connect to WebSocket: {e}[/red]")
            return False

    async def _disconnect(self):
        """Disconnect from WebSocket server."""
        if self._websocket:
            await self._websocket.close()
            self._connected = False

    async def send_event(self, event: SDKEvent):
        if not self._connected or not self._websocket:
            console.print("[red]Not connected[/red]")
            return False

        try:
            await self._websocket.send(self.codec.encode(event))
            return True
        except Exception as e:
            console.print(f"[red]Failed to send event: {e}[/red]")
            return False

    async def send_message(self, user_message: str) -> Optional[str]:
        """Send a user message and receive the agent's response."""
        if not self._connected or not self._websocket:
            console.print("[red]Not connected[/red]")
            return None

        try:
            await self._websocket.send(
                self.codec.encode(
                    SDKAgentTranscriptUpdateEvent(role="user", content=user_message)
                )
            )

            await self._websocket.send(self.codec.encode(SDKSystemLLMRequestEvent()))

            full_response = ""
            response_text = Text()

            with Live(
                Panel(
                    response_text,
                    title="[bold blue]Assistant[/bold blue]",
                    border_style="blue",
                ),
                console=console,
                refresh_per_second=10,
                transient=True,
            ) as live:
                while True:
                    try:
                        message = await asyncio.wait_for(
                            self._websocket.recv(), timeout=60.0
                        )
                        if isinstance(message, str):
                            message = message.encode()
                        event = self.codec.decode(message)

                        if isinstance(event, SDKAgentLLMResponseStartEvent):
                            pass

                        elif isinstance(event, SDKAgentLLMResponseChunkEvent):
                            chunk_text = event.text
                            full_response += chunk_text
                            response_text.append(chunk_text)
                            live.update(
                                Panel(
                                    response_text,
                                    title="[bold blue]Assistant[/bold blue]",
                                    border_style="blue",
                                )
                            )

                        elif isinstance(event, SDKAgentLLMResponseEndEvent):
                            break

                        elif isinstance(event, SDKAgentErrorEvent):
                            console.print(f"[red]Error: {event.message}[/red]")
                            return None

                        elif isinstance(event, SDKAgentSpeakEvent):
                            speak_text = event.text
                            full_response += speak_text
                            response_text.append(speak_text)
                            live.update(
                                Panel(
                                    response_text,
                                    title="[bold blue]Assistant[/bold blue]",
                                    border_style="blue",
                                )
                            )

                    except asyncio.TimeoutError:
                        console.print("[yellow]Response timeout[/yellow]")
                        break

            if full_response:
                console.print(
                    Panel(
                        Markdown(full_response),
                        title="[bold blue]Assistant[/bold blue]",
                        border_style="blue",
                    )
                )

                await self._websocket.send(
                    self.codec.encode(
                        SDKAgentTranscriptUpdateEvent(
                            role="assistant", content=full_response
                        )
                    )
                )

            return full_response

        except Exception as e:
            console.print(f"[red]Error sending message: {e}[/red]")
            return None

    async def receive_initial_message(self) -> Optional[str]:
        """Receive any initial message from the agent (like a greeting)."""
        if not self._connected or not self._websocket:
            return None

        try:
            full_response = ""
            response_text = Text()

            try:
                message = await asyncio.wait_for(self._websocket.recv(), timeout=2.0)

                if isinstance(message, str):
                    message = message.encode()

                event = self.codec.decode(message)

                if isinstance(event, SDKAgentSpeakEvent):
                    full_response = event.text

                elif isinstance(event, SDKAgentLLMResponseStartEvent):
                    with Live(
                        Panel(
                            response_text,
                            title="[bold blue]Assistant[/bold blue]",
                            border_style="blue",
                        ),
                        console=console,
                        refresh_per_second=10,
                        transient=True,
                    ) as live:
                        while True:
                            try:
                                msg = await asyncio.wait_for(
                                    self._websocket.recv(), timeout=30.0
                                )
                                if isinstance(msg, str):
                                    msg = msg.encode()
                                evt = self.codec.decode(msg)

                                if isinstance(evt, SDKAgentLLMResponseChunkEvent):
                                    chunk = evt.text
                                    full_response += chunk
                                    response_text.append(chunk)
                                    live.update(
                                        Panel(
                                            response_text,
                                            title="[bold blue]Assistant[/bold blue]",
                                            border_style="blue",
                                        )
                                    )
                                elif isinstance(evt, SDKAgentLLMResponseEndEvent):
                                    break
                            except asyncio.TimeoutError:
                                break

            except asyncio.TimeoutError:
                pass

            if full_response:
                console.print(
                    Panel(
                        Markdown(full_response),
                        title="[bold blue]Assistant[/bold blue]",
                        border_style="blue",
                    )
                )

                await self._websocket.send(
                    self.codec.encode(
                        SDKAgentTranscriptUpdateEvent(
                            role="assistant", content=full_response
                        )
                    )
                )

            return full_response if full_response else None

        except Exception:
            return None


async def chat_loop(client: ChatClient):
    """Main chat loop."""
    await client.send_event(SDKSystemUserJoinedEvent())
    await client.receive_initial_message()

    console.print(
        "[dim]Type your message and press Enter. Type 'exit' or 'quit' to leave.[/dim]\n"
    )

    while True:
        try:
            user_input = Prompt.ask("[bold green]You[/bold green]")

            if user_input.lower() in ["exit", "quit", "q"]:
                console.print("[dim]Goodbye![/dim]")
                break

            if not user_input.strip():
                continue

            await client.send_message(user_input)
            console.print()

        except KeyboardInterrupt:
            console.print("\n[dim]Interrupted. Goodbye![/dim]")
            break
        except EOFError:
            console.print("\n[dim]Goodbye![/dim]")



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from .types.get_agent_template_response import GetAgentTemplateResponse
from .types.post_agent_from_template_response import PostAgentFromTemplateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAgentTemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_agent_templates(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetAgentTemplateResponse]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetAgentTemplateResponse]
            Successful response
        """
        _response = self._client_wrapper.httpx_client.request(
            "agent/template",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAgentTemplateResponse,
                    construct_type(
                        type_=GetAgentTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_agent_from_template(
        self,
        *,
        agent_name: str,
        template_id: str,
        agent_description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PostAgentFromTemplateResponse]:
        """
        We have created templates for some common use cases. You can use these templates to create an agent. For getting list of templates, you can use the /agent/template endpoint. It will give you the list of templates with their description and id. You can pass the id of the template in the request body to create an agent from the template.

        Parameters
        ----------
        agent_name : str
            Name of the agent

        template_id : str
            ID of the template to use. You can get the list of templates with their description and id from the /agent/template endpoint.

        agent_description : typing.Optional[str]
            Description of the agent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostAgentFromTemplateResponse]
            Successful response
        """
        _response = self._client_wrapper.httpx_client.request(
            "agent/from-template",
            base_url=self._client_wrapper.get_environment().atoms,
            method="POST",
            json={
                "agentName": agent_name,
                "agentDescription": agent_description,
                "templateId": template_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostAgentFromTemplateResponse,
                    construct_type(
                        type_=PostAgentFromTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAgentTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_agent_templates(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetAgentTemplateResponse]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetAgentTemplateResponse]
            Successful response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "agent/template",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAgentTemplateResponse,
                    construct_type(
                        type_=GetAgentTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_agent_from_template(
        self,
        *,
        agent_name: str,
        template_id: str,
        agent_description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PostAgentFromTemplateResponse]:
        """
        We have created templates for some common use cases. You can use these templates to create an agent. For getting list of templates, you can use the /agent/template endpoint. It will give you the list of templates with their description and id. You can pass the id of the template in the request body to create an agent from the template.

        Parameters
        ----------
        agent_name : str
            Name of the agent

        template_id : str
            ID of the template to use. You can get the list of templates with their description and id from the /agent/template endpoint.

        agent_description : typing.Optional[str]
            Description of the agent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostAgentFromTemplateResponse]
            Successful response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "agent/from-template",
            base_url=self._client_wrapper.get_environment().atoms,
            method="POST",
            json={
                "agentName": agent_name,
                "agentDescription": agent_description,
                "templateId": template_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostAgentFromTemplateResponse,
                    construct_type(
                        type_=PostAgentFromTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel
from .lightning_v31tts_request_message_language import LightningV31TtsRequestMessageLanguage


class LightningV31TtsRequestMessage(UncheckedBaseModel):
    voice_id: str = pydantic.Field()
    """
    The ID of the voice to use
    """

    text: str = pydantic.Field()
    """
    The text to convert to speech
    """

    max_buffer_flush_ms: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum time (in ms) to wait for more input before generating output. It flushes when either this time is reached or enough input is received for optimal output—whichever comes first. This is useful for input streams. Defaults to 0
    """

    continue_: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="continue"),
        pydantic.Field(
            alias="continue",
            description="This setting controls whether the system should buffer and wait for more input after receiving the current one. If not set, it assumes no more input is coming.",
        ),
    ] = None
    flush: typing.Optional[bool] = pydantic.Field(default=None)
    """
    This setting controls whether the system should flush the current buffer.
    """

    complete_backoff_ms: typing.Optional[float] = pydantic.Field(default=None)
    """
    The time in ms to wait after the last chunk is sent before sending the complete response. Default is 4000ms. Maximum is 10000ms.
    """

    language: typing.Optional[LightningV31TtsRequestMessageLanguage] = pydantic.Field(default=None)
    """
    The language code, available options: `en`, `hi`
    """

    sample_rate: typing.Optional[int] = pydantic.Field(default=None)
    """
    Audio sample rate in Hz. Supported values: 8000, 16000, 24000, 44100
    """

    speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    Speaking speed multiplier
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# coding: utf-8

"""
    Agent Management API

    API for managing agents, their templates, and call logs

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBytes, StrictStr
from typing import Tuple, Union
from typing_extensions import Annotated
from smallestai.atoms.models.agent_id_delete200_response import AgentIdDelete200Response
from smallestai.atoms.models.knowledgebase_get200_response import KnowledgebaseGet200Response
from smallestai.atoms.models.knowledgebase_id_get200_response import KnowledgebaseIdGet200Response
from smallestai.atoms.models.knowledgebase_id_items_get200_response import KnowledgebaseIdItemsGet200Response
from smallestai.atoms.models.knowledgebase_id_items_upload_text_post_request import KnowledgebaseIdItemsUploadTextPostRequest
from smallestai.atoms.models.knowledgebase_post201_response import KnowledgebasePost201Response
from smallestai.atoms.models.knowledgebase_post_request import KnowledgebasePostRequest

from smallestai.atoms.api_client import ApiClient, RequestSerialized
from smallestai.atoms.api_response import ApiResponse
from smallestai.atoms.rest import RESTResponseType


class KnowledgeBaseApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def knowledgebase_get(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> KnowledgebaseGet200Response:
        """Get all knowledge bases

        Get all knowledge bases

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KnowledgebaseGet200Response",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def knowledgebase_get_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[KnowledgebaseGet200Response]:
        """Get all knowledge bases

        Get all knowledge bases

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KnowledgebaseGet200Response",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def knowledgebase_get_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get all knowledge bases

        Get all knowledge bases

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_get_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KnowledgebaseGet200Response",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _knowledgebase_get_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/knowledgebase',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def knowledgebase_id_delete(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AgentIdDelete200Response:
        """Delete a knowledge base

        Delete a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_delete_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def knowledgebase_id_delete_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AgentIdDelete200Response]:
        """Delete a knowledge base

        Delete a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_delete_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def knowledgebase_id_delete_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete a knowledge base

        Delete a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_delete_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _knowledgebase_id_delete_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/knowledgebase/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def knowledgebase_id_get(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> KnowledgebaseIdGet200Response:
        """Get a knowledge base

        Get a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_get_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KnowledgebaseIdGet200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def knowledgebase_id_get_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[KnowledgebaseIdGet200Response]:
        """Get a knowledge base

        Get a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_get_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KnowledgebaseIdGet200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def knowledgebase_id_get_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get a knowledge base

        Get a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_get_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KnowledgebaseIdGet200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _knowledgebase_id_get_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/knowledgebase/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def knowledgebase_id_items_get(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> KnowledgebaseIdItemsGet200Response:
        """Get all knowledge base items

        Get all knowledge base items

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_items_get_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KnowledgebaseIdItemsGet200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def knowledgebase_id_items_get_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[KnowledgebaseIdItemsGet200Response]:
        """Get all knowledge base items

        Get all knowledge base items

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_items_get_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KnowledgebaseIdItemsGet200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def knowledgebase_id_items_get_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get all knowledge base items

        Get all knowledge base items

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_items_get_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KnowledgebaseIdItemsGet200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _knowledgebase_id_items_get_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/knowledgebase/{id}/items',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def knowledgebase_id_items_upload_media_post(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        media: Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AgentIdDelete200Response:
        """Upload a media to a knowledge base

        Upload a media to a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param media: (required)
        :type media: bytearray
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_items_upload_media_post_serialize(
            id=id,
            media=media,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def knowledgebase_id_items_upload_media_post_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        media: Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AgentIdDelete200Response]:
        """Upload a media to a knowledge base

        Upload a media to a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param media: (required)
        :type media: bytearray
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_items_upload_media_post_serialize(
            id=id,
            media=media,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def knowledgebase_id_items_upload_media_post_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        media: Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Upload a media to a knowledge base

        Upload a media to a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param media: (required)
        :type media: bytearray
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_items_upload_media_post_serialize(
            id=id,
            media=media,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _knowledgebase_id_items_upload_media_post_serialize(
        self,
        id,
        media,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        if media is not None:
            _files['media'] = media
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'multipart/form-data'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/knowledgebase/{id}/items/upload-media',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def knowledgebase_id_items_upload_text_post(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        knowledgebase_id_items_upload_text_post_request: KnowledgebaseIdItemsUploadTextPostRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AgentIdDelete200Response:
        """Upload a text to a knowledge base

        Upload a text to a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param knowledgebase_id_items_upload_text_post_request: (required)
        :type knowledgebase_id_items_upload_text_post_request: KnowledgebaseIdItemsUploadTextPostRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_items_upload_text_post_serialize(
            id=id,
            knowledgebase_id_items_upload_text_post_request=knowledgebase_id_items_upload_text_post_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def knowledgebase_id_items_upload_text_post_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        knowledgebase_id_items_upload_text_post_request: KnowledgebaseIdItemsUploadTextPostRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AgentIdDelete200Response]:
        """Upload a text to a knowledge base

        Upload a text to a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param knowledgebase_id_items_upload_text_post_request: (required)
        :type knowledgebase_id_items_upload_text_post_request: KnowledgebaseIdItemsUploadTextPostRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_items_upload_text_post_serialize(
            id=id,
            knowledgebase_id_items_upload_text_post_request=knowledgebase_id_items_upload_text_post_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def knowledgebase_id_items_upload_text_post_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        knowledgebase_id_items_upload_text_post_request: KnowledgebaseIdItemsUploadTextPostRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Upload a text to a knowledge base

        Upload a text to a knowledge base

        :param id: The ID of the knowledge base (required)
        :type id: str
        :param knowledgebase_id_items_upload_text_post_request: (required)
        :type knowledgebase_id_items_upload_text_post_request: KnowledgebaseIdItemsUploadTextPostRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_id_items_upload_text_post_serialize(
            id=id,
            knowledgebase_id_items_upload_text_post_request=knowledgebase_id_items_upload_text_post_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _knowledgebase_id_items_upload_text_post_serialize(
        self,
        id,
        knowledgebase_id_items_upload_text_post_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if knowledgebase_id_items_upload_text_post_request is not None:
            _body_params = knowledgebase_id_items_upload_text_post_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/knowledgebase/{id}/items/upload-text',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def knowledgebase_knowledge_base_id_items_knowledge_base_item_id_delete(
        self,
        knowledge_base_id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        knowledge_base_item_id: Annotated[StrictStr, Field(description="The ID of the knowledge base item")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AgentIdDelete200Response:
        """Delete a knowledge base item

        Delete a knowledge base item

        :param knowledge_base_id: The ID of the knowledge base (required)
        :type knowledge_base_id: str
        :param knowledge_base_item_id: The ID of the knowledge base item (required)
        :type knowledge_base_item_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_knowledge_base_id_items_knowledge_base_item_id_delete_serialize(
            knowledge_base_id=knowledge_base_id,
            knowledge_base_item_id=knowledge_base_item_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def knowledgebase_knowledge_base_id_items_knowledge_base_item_id_delete_with_http_info(
        self,
        knowledge_base_id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        knowledge_base_item_id: Annotated[StrictStr, Field(description="The ID of the knowledge base item")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AgentIdDelete200Response]:
        """Delete a knowledge base item

        Delete a knowledge base item

        :param knowledge_base_id: The ID of the knowledge base (required)
        :type knowledge_base_id: str
        :param knowledge_base_item_id: The ID of the knowledge base item (required)
        :type knowledge_base_item_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_knowledge_base_id_items_knowledge_base_item_id_delete_serialize(
            knowledge_base_id=knowledge_base_id,
            knowledge_base_item_id=knowledge_base_item_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def knowledgebase_knowledge_base_id_items_knowledge_base_item_id_delete_without_preload_content(
        self,
        knowledge_base_id: Annotated[StrictStr, Field(description="The ID of the knowledge base")],
        knowledge_base_item_id: Annotated[StrictStr, Field(description="The ID of the knowledge base item")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete a knowledge base item

        Delete a knowledge base item

        :param knowledge_base_id: The ID of the knowledge base (required)
        :type knowledge_base_id: str
        :param knowledge_base_item_id: The ID of the knowledge base item (required)
        :type knowledge_base_item_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_knowledge_base_id_items_knowledge_base_item_id_delete_serialize(
            knowledge_base_id=knowledge_base_id,
            knowledge_base_item_id=knowledge_base_item_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AgentIdDelete200Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _knowledgebase_knowledge_base_id_items_knowledge_base_item_id_delete_serialize(
        self,
        knowledge_base_id,
        knowledge_base_item_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if knowledge_base_id is not None:
            _path_params['knowledgeBaseId'] = knowledge_base_id
        if knowledge_base_item_id is not None:
            _path_params['knowledgeBaseItemId'] = knowledge_base_item_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/knowledgebase/{knowledgeBaseId}/items/{knowledgeBaseItemId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def knowledgebase_post(
        self,
        knowledgebase_post_request: KnowledgebasePostRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> KnowledgebasePost201Response:
        """Create a knowledge base

        Create a knowledge base

        :param knowledgebase_post_request: (required)
        :type knowledgebase_post_request: KnowledgebasePostRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_post_serialize(
            knowledgebase_post_request=knowledgebase_post_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "KnowledgebasePost201Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def knowledgebase_post_with_http_info(
        self,
        knowledgebase_post_request: KnowledgebasePostRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[KnowledgebasePost201Response]:
        """Create a knowledge base

        Create a knowledge base

        :param knowledgebase_post_request: (required)
        :type knowledgebase_post_request: KnowledgebasePostRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_post_serialize(
            knowledgebase_post_request=knowledgebase_post_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "KnowledgebasePost201Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def knowledgebase_post_without_preload_content(
        self,
        knowledgebase_post_request: KnowledgebasePostRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create a knowledge base

        Create a knowledge base

        :param knowledgebase_post_request: (required)
        :type knowledgebase_post_request: KnowledgebasePostRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._knowledgebase_post_serialize(
            knowledgebase_post_request=knowledgebase_post_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "KnowledgebasePost201Response",
            '400': "BadRequestErrorResponse",
            '401': "UnauthorizedErrorReponse",
            '500': "InternalServerErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _knowledgebase_post_serialize(
        self,
        knowledgebase_post_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if knowledgebase_post_request is not None:
            _body_params = knowledgebase_post_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/knowledgebase',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





"""
Node - Base class for graph-based agent nodes.

Supports tree/graph structures with multiple children and parents.
"""

import asyncio
from typing import Optional, Set

from loguru import logger

from smallestai.atoms.agent.events import (
    SDKEvent,
    SDKSystemControlInterruptEvent,
    SDKSystemInitEvent,
)
from smallestai.atoms.agent.task_manager import TaskManager


class Node:
    """Base class for nodes in the graph.

    Nodes are the building blocks of the graph. They can be linked together to form a tree or DAG.
    Each node can have multiple children and parents (for graph structures).

    Nodes can be of different types:
    - Root: nodes that receive events from outside (no parents)
    - Processor: nodes that process events (have parents and children)
    - Leaf: nodes at the end of branches (no children, connect to sink)
    - Sink: special node that outputs to WebSocket
    """

    def __init__(self, name: Optional[str] = None, is_interruptible: bool = False):
        """
        Initialize the node.

        Args:
            name: Optional name for this node
        """
        self.name = name or self.__class__.__name__

        # Graph structure - support multiple parents and children
        self._parents: Set["Node"] = set()
        self._children: Set["Node"] = set()

        # Event queue for this node
        self._queue: asyncio.Queue = asyncio.Queue()

        # Task management
        self._running = False

        self._task_manager: Optional[TaskManager] = None

        self._process_event_task: Optional[asyncio.Task] = None

        self._is_interruptible = is_interruptible

    @property
    def is_interruptible(self) -> bool:
        """Get whether this node is interruptible"""
        return self._is_interruptible

    @property
    def task_manager(self) -> Optional[TaskManager]:
        """Get the task manager for this node"""
        if not self._running:
            return None
        if not self._task_manager:
            raise ValueError(f"Task manager not set for node {self.name}")
        return self._task_manager

    def add_child(self, child: "Node"):
        """Add a child node"""
        self._children.add(child)
        child._parents.add(self)
        logger.debug(f"Linked {self.name} -> {child.name}")

    def add_parent(self, parent: "Node"):
        """Add a parent node"""
        self._parents.add(parent)
        parent._children.add(self)
        logger.debug(f"Linked {parent.name} -> {self.name}")

    @property
    def children(self) -> Set["Node"]:
        """Get all child nodes"""
        return self._children

    @property
    def parents(self) -> Set["Node"]:
        """Get all parent nodes"""
        return self._parents

    @property
    def has_parents(self) -> bool:
        """Check if this node has parents"""
        return len(self._parents) > 0

    @property
    def has_children(self) -> bool:
        """Check if this node has children"""
        return len(self._children) > 0

    async def queue_event(self, event: SDKEvent):
        """
        Queue an event for processing by this node.

        Args:
            event: The event to queue
        """
        await self._queue.put(event)

    async def send_event(self, event: SDKEvent):
        """
        Send an event in the specified direction.

        Args:
            event: The event to send
            direction: The direction to send the event
        """
        for child in self._children:
            logger.trace(f"[{self.name}] Sending event to {child.name}")
            if isinstance(event, SDKSystemControlInterruptEvent):
                await child.process_event(event)
            else:
                await child.queue_event(event)

    async def _handle_interrupt(self):
        """Handle interrupt"""

        await self.__cancel_process_event_task()
        self._queue = asyncio.Queue()
        await self.__create_process_event_task()

    async def process_event(self, event: SDKEvent):
        """
        Process an event (override this in subclass).

        Args:
            event: The event to process
        """

        if isinstance(event, SDKSystemControlInterruptEvent) and self.is_interruptible:
            await self._handle_interrupt()

    async def __process_event_handler_loop(self):
        """Process event handler loop"""
        try:
            while self._running:
                event = await self._queue.get()

                if event is None:
                    break

                await self.process_event(event)
        except Exception as e:
            logger.exception(f"[{self.name}] Unexpected error in processing loop: {e}")

    async def __create_process_event_task(self):
        """Create a task to process events"""
        if self.task_manager:
            self._process_event_task = self.task_manager.create_task(
                self.__process_event_handler_loop(),
                name=f"{self.name}::process_event_handler_loop",
            )

    async def __cancel_process_event_task(self):
        """Cancel the task to process events"""
        if self.task_manager and self._process_event_task:
            await self.task_manager.cancel_task(self._process_event_task)
            self._process_event_task = None

    async def start(self, init_event: SDKSystemInitEvent, task_manager: TaskManager):
        """Start the node"""
        if self._running:
            logger.warning(f"[{self.name}] Already running")
            return

        self._running = True
        self._task_manager = task_manager
        await self.__create_process_event_task()
        logger.info(f"[{self.name}] Started")

    async def stop(self):
        """Stop the agent processor"""
        if not self._running:
            return

        self._running = False
        await self._queue.put(None)

        await self.__cancel_process_event_task()
        logger.info(f"[{self.name}] Stopped")

    def __repr__(self):
        return f"<Node: {self.name}>"



# This file was auto-generated by Fern from our API Definition.

import typing

GetConversationResponseDataLogsItemType = typing.Union[
    typing.Literal["telephony_inbound", "telephony_outbound", "chat"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel
from .lightning_speech_to_text_response_age import LightningSpeechToTextResponseAge
from .lightning_speech_to_text_response_emotions import LightningSpeechToTextResponseEmotions
from .lightning_speech_to_text_response_gender import LightningSpeechToTextResponseGender
from .lightning_speech_to_text_response_metadata import LightningSpeechToTextResponseMetadata
from .lightning_speech_to_text_response_utterances_item import LightningSpeechToTextResponseUtterancesItem
from .lightning_speech_to_text_response_words_item import LightningSpeechToTextResponseWordsItem


class LightningSpeechToTextResponse(UncheckedBaseModel):
    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status of the transcription request
    """

    text: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="transcription"),
        pydantic.Field(alias="transcription", description="The transcribed text from the audio file"),
    ] = None
    audio_length: typing.Optional[float] = pydantic.Field(default=None)
    """
    Duration of the audio file in seconds
    """

    words: typing.Optional[typing.List[LightningSpeechToTextResponseWordsItem]] = pydantic.Field(default=None)
    """
    Word-level timestamps in seconds.
    """

    utterances: typing.Optional[typing.List[LightningSpeechToTextResponseUtterancesItem]] = pydantic.Field(default=None)
    """
    List of utterances with start and end times
    """

    age: typing.Optional[LightningSpeechToTextResponseAge] = pydantic.Field(default=None)
    """
    Predicted age group of the speaker (e.g., infant, teenager, adult, old)
    """

    gender: typing.Optional[LightningSpeechToTextResponseGender] = pydantic.Field(default=None)
    """
    Predicted gender of the speaker if requested
    """

    emotions: typing.Optional[LightningSpeechToTextResponseEmotions] = pydantic.Field(default=None)
    """
    Predicted emotions of the speaker if requested
    """

    metadata: typing.Optional[LightningSpeechToTextResponseMetadata] = pydantic.Field(default=None)
    """
    Metadata about the transcription
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.unchecked_base_model import UncheckedBaseModel
from .workflow_graph_data_nodes_item_position import WorkflowGraphDataNodesItemPosition


class WorkflowGraphDataNodesItem(UncheckedBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the node
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type of the node (e.g., default_node, end_call, pre_call_api)
    """

    position: typing.Optional[WorkflowGraphDataNodesItemPosition] = None
    data: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    Node-specific data and configuration
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
